# maxclique.py

from pyomo.environ import *
from pyomo.opt import SolverStatus, TerminationCondition
import pyomo.opt

def create_model(n, E):
    model = ConcreteModel()
    model.Nodes = Set(initialize=range(n))
    model.Pairs = Set(initialize=[(i,j) for i in range(n) for j in range(n) if i!=j])
    A = [(i,j) for (i,j) in E]
    A.extend([(j,i) for (i,j) in E])
    model.Arcs = Set(initialize=A)
    model.MissingArcs = model.Pairs - model.Arcs
    model.x = Var(model.Nodes, within = Binary)
    def objective_function(model):
        return -sum(model.x[i] for i in model.Nodes)
    model.obj = Objective(rule = objective_function)
    def clique_constraint(model, i, j):
        return model.x[i] + model.x[j] <= 1
    model.clique = Constraint(model.MissingArcs, rule=clique_constraint)
    return model

# main
n = 5
E = [(1,2), (1,4), (2,4), (2,5), (3,5)]
instance = create_model(n, E)
solver = pyomo.opt.SolverFactory('cplex', solver_io = 'nl')
if solver is None:
    raise RuntimeError('solver not found')
solver.options['timelimit'] = 60
results = solver.solve(instance, keepfiles = False, tee = True)
if ((results.solver.status == SolverStatus.ok) and (results.solver.termination_condition == TerminationCondition.optimal)):
    # feasible and optimal
    instance.solutions.load_from(results)
    xstar = [instance.x[i].value for i in instance.Nodes]
    print('solved to optimality')
    print(xstar)
else:
    print('not solved to optimality')
    
