# sat.py

import sys
from pyomo.environ import *
from pyomo.opt import SolverStatus, TerminationCondition
import pyomo.opt


def readInstance(fileName):
    f = open(fileName, 'r')
    
    cnf = list()
    cnf.append(list())
    maxvar = 0

    for line in f.readlines():
        tokens = line.split()
        if len(tokens) != 0 and tokens[0] not in ("p", "c", "\t", " "):
            for tok in tokens:
                lit = int(tok)
                maxvar = max(maxvar, abs(lit))
                if lit == 0:
                    cnf.append(list())
                else:
                    cnf[-1].append(lit)
                    
    assert len(cnf[-1]) == 0
    cnf.pop()
    return (cnf, maxvar)

def create_model(cnf, numvars):
    model = ConcreteModel()
    m = len(cnf)
    n = numvars
    model.M = Set(initialize=range(m))
    model.N = Set(initialize=range(n))
    model.x = Var(model.N, within = Binary)
    def objective_function(model):
        return 0
    model.obj = Objective(rule = objective_function)
    def sat_constraint(model, i):
        expr = 0
        for j,val in enumerate(cnf[i]):
            if val > 0:
                expr = expr + model.x[j]
            elif val < 0:
                expr = expr + 1 - model.x[j]
        return expr >= 1
    model.sat = Constraint(model.M, rule=sat_constraint)
    return model


### main

if len(sys.argv) < 2:
    print "error, need SAT Dimacs-formatted file on cmd line"
    exit(1)

inputFile = sys.argv[1]
(cnf, maxvar) = readInstance(inputFile)
instance = create_model(cnf, maxvar)
print "------- CPLEX output -------"
solver = pyomo.opt.SolverFactory('cplex', solver_io = 'nl')
if solver is None:
    raise RuntimeError('solver not found')
solver.options['timelimit'] = 60
results = solver.solve(instance, keepfiles = False, tee = True)
print "----- end CPLEX output -----"
if ((results.solver.status == SolverStatus.ok) and (results.solver.termination_condition == TerminationCondition.optimal)):
    # feasible and optimal
    instance.solutions.load_from(results)
    xstar = [instance.x[i].value for i in instance.N]
    print 'instance is satisfiable: solution is', xstar
else:
    print 'instance is not satisfiable'
